#--------------------------------------------------------------------------------------------------------------------------
# Listening Server to read values from Growatt inverter via wifi-X (TCP Modbus receiver)
#
# This module parses the binary data blocks
#
# License:
# Feel free to use this code in your private or business environment or as example for your own code.
# Redistribution is only allowed with no changes.
# For any other kind of usage, please ask me.
# Ralf Amandi, Avarix GmbH
#
# 14.05.2021 Version 1.0
#--------------------------------------------------------------------------------------------------------------------------
from datetime import datetime
LoV_Id={}

def SetConfig(config):
	global LoV_Id
	LoV_Id = config["LoV_ID"]

def Fmt2(val):
	if (val<10):
		return "0"+str(val)
	else:
		return str(val)

# Reads the Datablock to json
def readFunc4(bin):
	v = {'FUNC':bin.func}
	energy={}
	ac = {"VOLT":[], "CURR":[], "POW":[]}
	mpp = {"VOLT":[], "CURR":[], "POW":[]}
	dc = {}
	limit = {}
	
	sysinfo = {}
	status = {}
	config = {}
	
	sysinfo['SERIAL']=bin.getStr(0,10) # Bis 30 Zeichen länge, Serial WiFi-X
	sysinfo['SERIAL-INV']=bin.getStr(30,10)
	dt = datetime (2000+bin.getB(60),bin.getB(61),bin.getB(62), bin.getB(63),bin.getB(64),bin.getB(65), bin.getB(66)) # 66 = Timezone
	status['TIMESTAMP']= dt.timestamp()
	status['DATETIME'] = dt.isoformat(sep=' ', timespec='seconds')
	
	#-- MPP 
	mpp['POWER'] = bin.getI32(73) / 10 # W
	mpp['COUNT'] = 2
	mpp['VOLT'].append(bin.getW(77) / 10) # Volt
	mpp['CURR'].append(bin.getW(79) / 10) # Amp
	mpp['POW'].append(bin.getI32(81) / 10) # W
	mpp['VOLT'].append(bin.getW(85) / 10) # Volt
	mpp['CURR'].append(bin.getW(87) / 10) # Amp
	mpp['POW'].append(bin.getI32(89) / 10) # W
	#---
	ac['POWER']=bin.getI32(117) / 10
	ac["FREQ"]=bin.getW(121) / 100
	cnt=0
	for i in range(3):
		if (bin.getW(123+i*8)>0):
			ac['VOLT'].append(bin.getW(123+i*8) / 10) # Volt
			ac['CURR'].append(bin.getW(125+i*8) / 10) # Amp
			ac['POW'].append(bin.getI32(127+i*8) / 10) # W
			cnt=cnt+1
	ac["COUNT"]=cnt
	ac["Vrs"]=bin.getI32(145)/10
	#--
	energy['TODAY']=bin.getI32(169)/10
	energy['TOTAL']=bin.getI32(173)/10
	energy['TOTALPV']=bin.getI32(177)/10
	energy['TODAYPV1']=bin.getI32(181)/10
	energy['TOTALPV1']=bin.getI32(185)/10
	energy['TODAYPV2']=bin.getI32(189)/10
	energy['TOTALPV2']=bin.getI32(193)/10
	energy['TST1']=bin.getI32(165)/10
	energy['TST2']=bin.getI32(197)/10

	dc['ISOLATION']=bin.getW(245) #KOHM
	status['INV_TEMP']=bin.getW(257)/10
	status['Temp2']=bin.getW(259)/10
	status['Temp3']=bin.getW(261)/10
	status['Temp4']=bin.getW(263)/10
	status['Temp5']=bin.getW(265)/10
	dc['BusVolt']=bin.getW(267)/10
	ac['Delay']=bin.getW(301)
	#--
	v['SYSINFO']=sysinfo
	v['STATUS']=status
	v['ENERGY']=energy
	v["AC"]=ac
	v["DC"]=dc
	v['MPP']=mpp
	#v['LIMIT']=limit
	v['CONFIG']=config

	return v

def readFunc3(bin):
	v = {'FUNC':bin.func}
	v['SERIAL']=bin.getStr(0,10) # max 30 chars
	v['SERIAL-INV']=bin.getStr(30,10) # max 30 chars
	v['DATETIME'] = "20"+Fmt2(bin.getB(60))+"-"+Fmt2(bin.getB(61))+"-"+Fmt2(bin.getB(62))+" "+Fmt2(bin.getB(63))+":"+Fmt2(bin.getB(64))+":"+Fmt2(bin.getB(65))# + 1 Byte Zeitzone
	v['VERS-FIRMWARE1']=bin.getStr(89,5) 
	v['VERS-COMM']=bin.getStr(95,4)+"-"+str(bin.getW(99)) 
	v['SERIAL-INV-AGAIN']=bin.getStr(117,10) # max 30 chars
	v['TYPE']=bin.getStr(139,16) # max 30 chars
	v['DATETIME2'] = Fmt2(bin.getW(161))+"-"+Fmt2(bin.getW(163))+"-"+Fmt2(bin.getB(165))+" "+Fmt2(bin.getB(167))+":"+Fmt2(bin.getB(169))+":"+Fmt2(bin.getB(171))
	v['VERS-FIRMWARE2']=bin.getStr(235,8) 
	d=[]
	for i in range( 254): 	#int((575-67) /2):
		d.append(bin.getW(67+i*2))
	v["tst"]=d
	return v
	
def readFunc80(bin):
	v = {'FUNC':bin.func}
	v['SERIAL']=bin.getStr(0,10) # max 30 chars
	v['SERIAL-INV']=bin.getStr(30,10) # max 30 chars
	v['DATETIME'] = "20"+Fmt2(bin.getB(60))+"-"+Fmt2(bin.getB(61))+"-"+Fmt2(bin.getB(62))+" "+Fmt2(bin.getB(63))+":"+Fmt2(bin.getB(64))+":"+Fmt2(bin.getB(65))# + 1 Byte Zeitzone
	d=[]
	for i in range(61):
		d.append(bin.getW(67+i*2))
	v["tst"]=d
	return v

def readReceipt(bin):
	v = {'FUNC':bin.func}
	
	if (bin.len() == 3):
		v['ACK']=str(bin.getB(0))
	else:
		v['ERROR']="LEN unexpected "+str(bin.len())
	return v

#===
# Datenstrings, werden über eine Id referenziert
def readFuncStrDataReq(bin):
	v = {'FUNC':bin.func}
	v['SERIAL']=bin.getStr(0,10) # max 30 chars
	if (bin.datalen == 36):
		v["REQUEST"]=str(bin.getW(30))+" - "+str(bin.getW(32))
	else:
		v['ERROR']="LEN unexpected "+str(bin.len())
	return v

def readAcknowledge(bin):
	v = {'FUNC':bin.func}
	if (bin.len() == 35):
		v['ID'] = str(bin.getW(30))
		v['ACK']=str(bin.getB(32))
	else:
		v['ERROR']="LEN unexpected "+str(bin.len())
	return v
		
def readFuncStrData(bin):
	global LoV_Id
	dl = bin.len()
	v = {'FUNC':bin.func}
	#print("F25:",bin.func,dl) 
	id = str(bin.getW(30))
	v['SERIAL']=bin.getStr(0,10) # max 30 chars
	if (dl==36 and bin.getW(32)>0):
		# Request aus Cloud 
		v["REQUEST"]=id+" - "+str(bin.getW(32))
	else:
		v['ID'] = id
		try:
			n = LoV_Id[id]
		except:
			n = id
		v['NAME'] = n
	
		if (dl<=34): # ?
			v["STATUS"]="NONE"
		elif (dl==35): # Statusbyte?
			v["STATUS"]=bin.getB(32)
		else: # Daten
			l = bin.getW(32)
			if (36+l>dl):
				v['ERR']="NEED: "+str(l)+", HAS"+str(dl-36)
			else:
				v['VALUE'] = bin.getStr(34,l)
	return v
